/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package jsr203.nio2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.channels.FileChannel;
import java.nio.file.FileSystem;
import java.nio.file.FileSystems;
import java.nio.file.Path;

/**
 *
 * @author mstoynov
 */
public class CopyAFile {

    public static void main(String... args) throws IOException {
        String from = "source.docx";
        String toIO = "destIO.docx";
        String toNIO = "destNIO.docx";
        String toNIO2 = "destNIO2.docx";

        copyIO(from, toIO);
        copyNIO(from, toNIO);
        copyNIO2(from, toNIO2);
    }

    public static void copyNIO2(String FileName, String toFileName) throws IOException {
        FileSystem local = FileSystems.getDefault();
        Path from = local.getPath(FileName);
        Path to = local.getPath(toFileName);
        from.copyTo(to);
    }

    public static void copyIO(String fromFileName, String toFileName)
            throws IOException {
        File fromFile = new File(fromFileName);
        File toFile = new File(toFileName);

        if (!fromFile.exists()) {
            throw new IOException("FileCopy: " + "no such source file: "
                    + fromFileName);
        }
        if (!fromFile.isFile()) {
            throw new IOException("FileCopy: " + "can't copy directory: "
                    + fromFileName);
        }
        if (!fromFile.canRead()) {
            throw new IOException("FileCopy: " + "source file is unreadable: "
                    + fromFileName);
        }

        if (toFile.isDirectory()) {
            toFile = new File(toFile, fromFile.getName());
        }

        if (toFile.exists()) {
            if (!toFile.canWrite()) {
                throw new IOException("FileCopy: "
                        + "destination file is unwriteable: " + toFileName);
            }
            System.out.print("Overwrite existing file " + toFile.getName()
                    + "? (Y/N): ");
            System.out.flush();
            BufferedReader in = new BufferedReader(new InputStreamReader(
                    System.in));
            String response = in.readLine();
            if (!response.equals("Y") && !response.equals("y")) {
                throw new IOException("FileCopy: "
                        + "existing file was not overwritten.");
            }
        } else {
            String parent = toFile.getParent();
            if (parent == null) {
                parent = System.getProperty("user.dir");
            }
            File dir = new File(parent);
            if (!dir.exists()) {
                throw new IOException("FileCopy: "
                        + "destination directory doesn't exist: " + parent);
            }
            if (dir.isFile()) {
                throw new IOException("FileCopy: "
                        + "destination is not a directory: " + parent);
            }
            if (!dir.canWrite()) {
                throw new IOException("FileCopy: "
                        + "destination directory is unwriteable: " + parent);
            }
        }

        FileInputStream from = null;
        FileOutputStream to = null;
        try {
            from = new FileInputStream(fromFile);
            to = new FileOutputStream(toFile);
            byte[] buffer = new byte[4096];
            int bytesRead;

            while ((bytesRead = from.read(buffer)) != -1) {
                to.write(buffer, 0, bytesRead); // write
            }
        } finally {
            if (from != null) {
                try {
                    from.close();
                } catch (IOException e) {
                    ;
                }
            }
            if (to != null) {
                try {
                    to.close();
                } catch (IOException e) {
                    ;
                }
            }
        }
    }

    public static void copyNIO(String fromFileName, String toFileName) throws IOException {
        // Getting file channels
        FileChannel in = new FileInputStream(fromFileName).getChannel();
        FileChannel out = new FileOutputStream(toFileName).getChannel();

        // JavaVM does its best to do this as native I/O operations.
        in.transferTo(0, in.size(), out);

        // Closing file channels will close corresponding stream objects as well.
        out.close();
        in.close();
    }
}
